/*
    Copyright 2002 Cyril Picard

    This file is part of the GEDCOMParser library 
    (developed within the Genealogy Free Software Tools project).

    The GEDCOMParser library is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    The GEDCOMParser library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with the GEDCOMParser library ; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/
#ifndef _GEDCOMPARSER_REPOSITORY_RECORD_HH_
#define _GEDCOMPARSER_REPOSITORY_RECORD_HH_

#include <map>
#include <string>

#include "GEDCOMParser/AddressManager.hh"
#include "GEDCOMParser/AttachmentManager.hh"
#include "GEDCOMParser/AutomatedRecordIdManager.hh"
#include "GEDCOMParser/ChangelogManager.hh"
#include "GEDCOMParser/UserReferenceManager.hh"
#include "GEDCOMParser/Visitor/GEDCOMElement.hh"

///
namespace GEDCOMParser
{
  class GEDCOMVisitor;
  /// @memo Implements the GEDCOM 5.5 REPOSITORY_RECORD structure   
  class RepositoryRecord:public GEDCOMElement,
    ///
    public AttachmentManager,
    public AutomatedRecordIdManager,
    public AddressManager,
    public ChangelogManager, public UserReferenceManager
  {
  public:
    RepositoryRecord (std::string const &id = "");
      virtual ~ RepositoryRecord (void)
    {
    };
    ///@name Accessors (set) 
    //@{    
    ///
    void setId (std::string const &);
    ///
    void setName (std::string const &);
    //@}

    ///@name Accessors (get)
    //@{    
    ///
    virtual std::string const &getId (void) const;
    ///
    virtual std::string const &getName (void) const;
    //@}
    virtual void Accept (GEDCOMParser::GEDCOMVisitor *);
  protected:
      std::string _id;
      std::string _name;
  };
  ///@name typedefs
  //@{
  /// @doc handling of many repository records
  typedef std::map < std::string,
    SmartPtr < RepositoryRecord > >RepositoryRecords_t;
  //@}
};
#endif
